
{******************************************}
{                                          }
{             FastReport VCL               }
{            EMF to PDF Export             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxEMFtoPDFExport;

interface

{$I frx.inc}

uses Windows, Graphics, Classes, frxExportHelpers, frxEMFAbstractExport,
  frxEMFFormat, frxExportPDFHelpers, frxClass, frxAnaliticGeometry, frxUtils;

type
  TBezierResult = record
    P0, P1, P2, P3: TfrxPoint;
  end;

  TPDFDeviceContext = class (TDeviceContext);

  TEMFtoPDFExport = class (TEMFAbstractExport)
  private
    FForceMitterLineJoin: Boolean;
    FForceButtLineCap: Boolean;
    FForceNullBrush: Boolean;
    FTransparency: Boolean;
    FForceAnsi: Boolean;
    FClipped: Boolean;
    FPictureDPI: Integer;
    FPrecision: Integer;

    procedure Put(const S: AnsiString);
    procedure PutCRLF(const S: AnsiString); {$IFDEF Delphi12} overload;
    procedure PutCRLF(const S: String); overload; {$ENDIF}
    procedure PutLF(const S: AnsiString); {$IFDEF Delphi12} overload;
    procedure PutLF(const S: String); overload; {$ENDIF}

    function pdfSize(emfSize: Extended): Extended;

    function pdfFrxPoint(emfP: TPoint): TfrxPoint; overload;
    function pdfFrxPoint(emfSP: TSmallPoint): TfrxPoint; overload;
    function pdfFrxPoint(emfDP: TfrxPoint): TfrxPoint; overload;

    function pdfFrxRect(emfR: TRect): TfrxRect;

//    function emfSize2Str(emfSize: Extended): String;

    function emfPoint2Str(emfP: TPoint): String; overload;
    function emfPoint2Str(emfSP: TSmallPoint): String; overload;
    function emfPoint2Str(emfFP: TfrxPoint): String; overload;

    function emfRect2Str(emfR: TRect): String;

    function EvenOdd: String;
    function IsNullBrush: Boolean;
    function IsNullPen: Boolean;
    procedure ReferenceToImageXObject(PicIndex: Integer; pdfRect: TfrxRect);

    function BezierCurve(Center, Radius: TfrxPoint; startAngle, arcAngle: Double): TBezierResult;
    procedure cmd_AngleArc(Center, Radius: TfrxPoint; StartAngle, SweepAngle: Single);
    procedure cmd_RoundRect(l, t, r, b, rx, ry: Extended);
    procedure cmdPathPainting(Options: integer);
    procedure cmdPathParams(Options: integer);
    procedure cmdSetClippingPath;
    procedure cmdCloseSubpath;
    procedure cmdAppendAngleArcToPath(AngleArc: TEMRAngleArc);
    procedure cmdAppendArcToPath(Arc: TEMRArc);
    procedure cmdAppendPieToPath(Pie: TEMRPie);
    procedure cmdAppendEllipsToPath(emfRect: TRect);
    procedure cmdAppendRectangleToPath(emfRect: TRect; IsCompartible: Boolean = False);
    procedure cmdAppendEMFRectangleToPath(emfRect: TRect);
    procedure cmdAppendRoundRectToPath(emfRect: TRect; emfCorners: TSize);

    procedure cmdMoveTo(X, Y: extended); overload;
    procedure cmdMoveTo(emfP: TPoint); overload;
    procedure cmdMoveTo(emfSP: TSmallPoint); overload;
    procedure cmdMoveTo(emfFP: TfrxPoint); overload;

    procedure cmdLineTo(X, Y: extended); overload;
    procedure cmdLineTo(emfP: TPoint); overload;
    procedure cmdLineTo(emfSP: TSmallPoint); overload;
    procedure cmdLineTo(emfDP: TfrxPoint); overload;
    procedure cmdSetLineDashPattern(PenStyle: LongWord; Width: Extended);
    procedure cmdSetStrokeColor(Color: TColor);
    procedure cmdSetFillColor(Color: TColor);
    procedure cmdSetLineWidth(Width: Extended); overload;
    procedure cmdSetLineWidth(PDFWidth: String); overload;
    procedure cmdSetMiterLimit(MiterLimit: Extended);
    procedure cmdSetLineCap(PenEndCap: Integer);
    procedure cmdSetLineJoin(PenLineJoin: Integer);
    procedure cmdAppendCurvedSegment2final(emfSP1, emfSP3: TSmallPoint); overload;
    procedure cmdAppendCurvedSegment2final(emfP1, emfP3: TPoint); overload;
    procedure cmdAppendCurvedSegment3(emfSP1, emfSP2, emfSP3: TSmallPoint); overload;
    procedure cmdAppendCurvedSegment3(emfP1, emfP2, emfP3: TPoint); overload;
    procedure cmdAppendCurvedSegment3(emfDP1, emfDP2, emfDP3: TfrxPoint); overload;
    procedure cmdPolyBezier(Options: integer = 0);
    procedure cmdPolyBezier16(Options: integer = 0);
    procedure cmdPolyLine(Options: integer = 0);
    procedure cmdPolyLine16(Options: integer = 0);
    procedure cmdPolyPolyLine(Options: integer = 0);
    procedure cmdPolyPolyLine16(Options: integer = 0);

    procedure cmdCreateExtSelectClipRgn;

    procedure cmdSaveGraphicsState;
    procedure cmdRestoreGraphicsState;

    procedure cmdBitmap(emfRect: TRect; dwRop: LongWord; EMRBitmap: TEMRBitmap);
    procedure BitmapSave(emfRect: TRect; Bitmap: TBitmap);
    procedure cmdBitmap32(emfRect: TRect; dwRop: LongWord; EMRBitmap: TEMRBitmap);

    procedure cmdTranslationAndScaling(Sx, Sy, Tx, Ty: Extended);
  protected
    FPDFRect: TfrxRect;
    FLastClipRect: TRect;
    FEMFtoPDFFactor: TfrxPoint;
    FPOH: TPDFObjectsHelper;
    FRotation2D: TRotation2D;
    FRealizationList: TStringList;
    FqQBalance: Integer;

    procedure Comment(CommentString: String = ''); override;
    procedure CommentAboutRealization;
    procedure CommentTextRect(rtl: TRect; Color: TColor = clRed);
    procedure RealizationListFill(RealizedCommands: array of String);
    function NormalizeRect(const Rect: TRect): TRect;
    procedure DCCreate; override;
    function FontCreate: TEMFFont; override;
    function PDFDeviceContext: TPDFDeviceContext;
    function IsSameCharacterWidth(FontName: string): Boolean;

    procedure DrawFontLines(FontSize: Double; TextPosition: TfrxPoint; TextWidth: Extended);
    procedure DrawFigureStart;
    procedure DrawFigureFinish(Options: integer);
    function FillStrokeOptions(Options: integer): integer;

    procedure DoEMR_AngleArc; override;
    procedure DoEMR_Arc; override;
    procedure DoEMR_AlphaBlend; override;
    procedure DoEMR_BitBlt; override;
    procedure DoEMR_CloseFigure; override;
    procedure DoEMR_Ellipse; override;
    procedure DoEMR_EoF; override;
    procedure DoEMR_ExtSelectClipRgn; override;
    procedure DoEMR_ExtTextOutW; override;
    procedure DoEMR_FillPath; override;
    procedure DoEMR_FillRgn; override;
    procedure DoEMR_Header; override;
    procedure DoEMR_IntersectClipRect; override;
    procedure DoEMR_LineTo; override;
    procedure DoEMR_MaskBlt; override;
    procedure DoEMR_MoveToEx; override;
    procedure DoEMR_Pie; override;
    procedure DoEMR_PolyBezier; override;
    procedure DoEMR_PolyBezier16; override;
    procedure DoEMR_PolyBezierTo; override;
    procedure DoEMR_PolyBezierTo16; override;
    procedure DoEMR_PolyDraw; override;
    procedure DoEMR_PolyDraw16; override;
    procedure DoEMR_Polygon; override;
    procedure DoEMR_Polygon16; override;
    procedure DoEMR_Polyline; override;
    procedure DoEMR_Polyline16; override;
    procedure DoEMR_PolylineTo; override;
    procedure DoEMR_PolylineTo16; override;
    procedure DoEMR_PolyPolygon; override;
    procedure DoEMR_PolyPolygon16; override;
    procedure DoEMR_PolyPolyline; override;
    procedure DoEMR_PolyPolyline16; override;
    procedure DoEMR_Rectangle; override;
    procedure DoEMR_RestoreDC; override;
    procedure DoEMR_RoundRect; override;
    procedure DoEMR_SaveDC; override;
    procedure DoEMR_SelectClipPath; override;
    procedure DoEMR_StretchBlt; override;
    procedure DoEMR_StretchDIBits; override;
    procedure DoEMR_StrokeAndFillPath; override;
    procedure DoEMR_StrokePath; override;
    procedure DoEMR_TransparentBlt; override;

    procedure DoStart; override;
    procedure DoFinish; override;
  public
    constructor Create(InStream, OutStream: TStream; APDFRect: TfrxRect; APOH: TPDFObjectsHelper);
    destructor Destroy; override;

    property ForceMitterLineJoin: Boolean read FForceMitterLineJoin write FForceMitterLineJoin;
    property ForceButtLineCap: Boolean write FForceButtLineCap;
    property ForceNullBrush: Boolean write FForceNullBrush;
    property Transparency: Boolean write FTransparency;
    property ForceAnsi: Boolean write FForceAnsi;
    property Clipped: Boolean write FClipped;
    property PictureDPI: Integer write FPictureDPI;
    property Precision: Integer write FPrecision;
  end;

implementation
end.

