unit frxComCtrls;
{$I frx.inc}
interface

uses
  {$IFNDEF NONWINFPC}Windows, CommCtrl,{$ENDIF}
  {$IFDEF FPC}LCLType, LMessages, LCLIntf, LazHelper, LazarusPackageIntf,{$ENDIF}
  Messages, SysUtils, Variants, Classes, Graphics,
  Controls, ComCtrls, frxDPIAwareBaseControls, frxUtils, frxCtrls, ImgList, frxDPIAwareInt;

type
  TfrxFilterAlign = (fraLeft, fraRight, fraWidth);
  TfrxToolAlign = (ftaTop, ftaBottom);
  TfrxTreeSortType = (dtsUnsorted, dtsAscending, dtsDescending);

  TfrxDataNode = class(TObject)
  private
    FParent: TfrxDataNode;
    FFirstChild: TfrxDataNode;
    FRSibling: TfrxDataNode;
    FLSibling: TfrxDataNode;
    FData: TObject;
    FImageIndex: Integer;
    FSelectedIndex: Integer;
    FOriginalNode: TTreeNode;
    FText: String;
  public
    destructor Destroy; override;
    procedure Clear;
  end;

  TfrxTreeNodeClass = class of TfrxTreeNode;

  TfrxTreeNode = class(TTreeNode)
  private
    FDataNode: TfrxDataNode;
    FNeedFixUp: Boolean;
  public
    constructor CreateWithData(AOwner: TTreeNodes; aDataNode: TfrxDataNode); overload;
    destructor Destroy; override;
    procedure MoveTo(Destination: TTreeNode; Mode: TNodeAttachMode); override;
  end;

  TfrxTreeNodes = class(TTreeNodes)
  protected
     procedure DefineProperties(Filer: TFiler); override;
  public
     procedure Assign(Source: TPersistent); override;
  end;

  TfrxTVFilterCompareEvent = function(Sender: TObject; Node: TTreeNode; const aFilter: string): Boolean of object;

  TfrxTreeView = class(TTreeView, IfrxDPIAwareControl)
  private
    FIsUpdating: Boolean;
    FOnEditedNew: TTVEditedEvent;
    FRootNode: TfrxDataNode;
    FLastFilter: String;
    FOnFilterCompare: TfrxTVFilterCompareEvent;
    FOnExpandButtonClick: TNotifyEvent;
    FExpandLevel: Integer;
    FDrawExpandButton: Boolean;
    FExpandButtonState: Boolean;
    FExpandButtonActive: Boolean;
    FExpandButtonWidth: Integer;
    FExpandButtonAlign: TfrxToolAlign;
    FExpandButtonRect: TRect;
    FCurrentPPI: Integer;
    procedure SetDrawExpandButton(const Value: Boolean);
    procedure SetExpandButtonState(const Value: Boolean);
    procedure InvalidateExpandButton(HideButton: Boolean = False);
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
{$IFDEF FPC}
    procedure WMMouseWheel(var Message: TLMMouseEvent); message LM_MOUSEWHEEL;
{$ELSE}
    procedure WMMouseWheel(var Message: TWMMouseWheel); message WM_MOUSEWHEEL;
{$ENDIF}
    procedure WMDpiChanged(var Message: TMessage); message FRX_WM_DPICHANGED_AFTERPARENT;
    procedure SetExpandButtonAlign(const Value: TfrxToolAlign);
    procedure UpdateExpandButton;
    procedure DrawArrow;
  protected
    function CreateNode: TTreeNode; override;
    function CreateNodes: TTreeNodes; override;
    function CanChange(Node: TTreeNode): Boolean; override;
    function CanCollapse(Node: TTreeNode): Boolean; override;
    procedure Change(Node: TTreeNode); override;
    procedure Delete(Node: TTreeNode); override;
    procedure Added(Node: TTreeNode); override;
    procedure DoEdit(Sender: TObject; Node: TTreeNode; var S: string);
    procedure DoPPIChanged(aNewPPI: Integer); virtual;
    procedure DestroyWnd; override;
    function IsCustomDrawn(Target: TCustomDrawTarget; Stage: TCustomDrawStage): Boolean; override;
    function CustomDraw(const ARect: TRect; Stage: TCustomDrawStage): Boolean; override;
    function CustomDrawItem(Node: TTreeNode; State: TCustomDrawState;
      Stage: TCustomDrawStage; var PaintImages: Boolean): Boolean; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure Resize; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ResetFilter;
    procedure Filter(const aFilter: string);
    procedure UpdateFilter;
    property ExpandLevel: Integer read FExpandLevel write FExpandLevel;
    property DrawExpandButton: Boolean read FDrawExpandButton write SetDrawExpandButton;
    property ExpandButtonState: Boolean read FExpandButtonState write SetExpandButtonState;
    property ExpandButtonAlign: TfrxToolAlign read FExpandButtonAlign write SetExpandButtonAlign;
  published
    property OnEdited: TTVEditedEvent read FOnEditedNew write FOnEditedNew;
    property OnFilterCompare: TfrxTVFilterCompareEvent read FOnFilterCompare write FOnFilterCompare;
    property OnExpandButtonClick: TNotifyEvent read FOnExpandButtonClick write FOnExpandButtonClick;
  end;

  TfrxToolPanel = class;

  TfrxToolButttonStyle = (fbsButton, fbsCheckButton, fbsRadioButton, fbsDropDownButton);
  TfrxToolButttonKind = (fbkButton, fbkSeparator);

  TfrxToolPanelCustomButton = class(TGraphicControl)
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TfrxToolPanelSeparator = class(TfrxToolPanelCustomButton)
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  { TfrxToolPanelButton }

  TfrxToolPanelButton = class(TfrxToolPanelCustomButton)
  private
    FIsActive: Boolean;
    FIsDown: Boolean;
    FImageIndex: Integer;
    FButtonStyle: TfrxToolButttonStyle;
    FGroup: Integer;
    function GetParentPanel: TfrxToolPanel;
    procedure SetButtonStyle(const Value: TfrxToolButttonStyle);
    procedure SetIsDown(const Value: Boolean);
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  protected
    procedure Paint; override;
    function GetArrowWidth: Integer;
  public
    constructor Create(AOwner: TComponent); override;
    //function IsDown: Boolean;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
{$IFDEF FPC}
    procedure ChangeBounds(ALeft, ATop, AWidth, AHeight: integer; KeepBase: boolean); override;
{$ENDIF}
    property ButtonStyle: TfrxToolButttonStyle read FButtonStyle write SetButtonStyle;
    property ImageIndex: Integer read FImageIndex write FImageIndex;
    property IsDown: Boolean read FIsDown write SetIsDown;
  end;

  TfrxToolPanel = class(TfrxDPIAwarePanel)
  private
    FButtons: TList;
    FImageList: TCustomImageList;
    FOnBtnClick: TNotifyEvent;
    procedure SetImageList(const Value: TCustomImageList);
    procedure ResetRadioGroup(GroupIndex: Integer);
  protected
    function CalcButtonsWidth: Integer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AddButton(ImageIndex: Integer; HintText: String = ''; Tag: NativeInt = -1; ButtonStyle: TfrxToolButttonStyle = fbsButton; Group: Integer = -1): TfrxToolPanelButton;
    function AddCustomButton(aKind: TfrxToolButttonKind): TfrxToolPanelCustomButton;
    procedure SetBounds(aLeft, aTop, aWidth, aHeight: integer); override;
{$IFDEF FPC}
    procedure ChangeBounds(ALeft, ATop, AWidth, AHeight: integer; KeepBase: boolean); override;
{$ENDIF}
    function GetArrowWidth: Integer;
    procedure Paint; override;
    procedure RealignButtons;
    property ImageList: TCustomImageList read FImageList write SetImageList;
    property OnBtnClick: TNotifyEvent read FOnBtnClick write FOnBtnClick;
  end;

  TfrxToolWithFilterPanel = class(TfrxDPIAwarePanel)
  private
    FToolPanel: TfrxToolPanel;
    FPanel: TfrxDPIAwarePanel;
    FFilter: TfrxFilterEdit;
    FFileterMinWidth: Integer;
    FFileterMaxWidth: Integer;
    FFilterActiveImageIndex: Integer;
    FFilterUnactiveImageIndex: Integer;
    FFilterAlign: TfrxFilterAlign;
    FToolPanelAlign: TfrxToolAlign;
    procedure SetFilterVisible(const Value: Boolean);
    function GetFilterVisible: Boolean;
    function GetToolVisible: Boolean;
    procedure SetToolVisible(const Value: Boolean);
    function GetImageList: TCustomImageList;
    procedure SetImageList(const Value: TCustomImageList);
    procedure SetFilterActiveImageIndex(const Value: Integer);
    procedure SetFilterUnactiveImageIndex(const Value: Integer);
    procedure SetFileterMaxWidth(const Value: Integer);
    procedure SetFileterMinWidth(const Value: Integer);
    procedure ExpandButtonClick(Sender: TObject); virtual;
    procedure SetFilterAlign(const Value: TfrxFilterAlign);
    procedure SetToolPanelAlign(const Value: TfrxToolAlign); virtual;
    procedure SetFilterColor(const Value: TColor);
    function GetFilterColor: TColor;
    procedure SetOnFilterChanged(const Value: TNotifyEvent);
  protected
    procedure UpdateSize; virtual;
    function CalcButtonsWidth: Integer;
    procedure Resize; override;
{$IFDEF FPC}
    procedure DoOnResize; override;
{$ENDIF}
    procedure SetParent(AParent: TWinControl); override;
    procedure DoPPIChanged(aNewPPI: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property FilterColor: TColor read GetFilterColor write SetFilterColor;
    property FilterEdit: TfrxFilterEdit read FFilter;
    property OnFilterChanged: TNotifyEvent write SetOnFilterChanged;
    property FilterVisible: Boolean read GetFilterVisible write SetFilterVisible;
    property FilterActiveImageIndex: Integer read FFilterActiveImageIndex write SetFilterActiveImageIndex;
    property FilterUnactiveImageIndex: Integer read FFilterUnactiveImageIndex write SetFilterUnactiveImageIndex;
    property FileterMinWidth: Integer read FFileterMinWidth write SetFileterMinWidth;
    property FileterMaxWidth: Integer read FFileterMaxWidth write SetFileterMaxWidth;
    property FilterAlign: TfrxFilterAlign read FFilterAlign write SetFilterAlign;
    property ToolPanel: TfrxToolPanel read FToolPanel;
    property ToolVisible: Boolean read GetToolVisible write SetToolVisible;
    property ToolImageList: TCustomImageList read GetImageList write SetImageList;
    property ToolPanelAlign: TfrxToolAlign read FToolPanelAlign write SetToolPanelAlign;
  end;

  { TfrxTreePanel }

  TfrxTreePanel = class(TfrxToolWithFilterPanel)
  private
    FTreeView: TfrxTreeView;
    procedure ExpandButtonClick(Sender: TObject); override;
    procedure SetToolPanelAlign(const Value: TfrxToolAlign); override;
  protected
    procedure UpdateSize; override;
    procedure SetParent(AParent: TWinControl); override;
    procedure EditChange(Sender: TObject); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property TreeView: TfrxTreeView read FTreeView;
  end;


implementation
end.
