
{******************************************}
{                                          }
{             FastReport VCL               }
{                Helpers                   }
{                                          }
{         Copyright (c) 1998-2022          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxHelpers;

{$I frx.inc}

interface

uses
  Classes, SysUtils,
  frxXML;

type
  TOwnObjList = class(TList)
  private
    FOwnsObjects: Boolean;
  protected
    function GetItem(Index: Integer): TObject;
  public
    constructor Create(AOwnsObjects: Boolean = True);
    procedure Clear; override;
    procedure AddNonZero(Item: Pointer);
    procedure Assign(SourceList: TList);

    property Items[Index: Integer]: TObject read GetItem; default;
  end;

  TValueDlm = (vdUnknown, vdApostrophe, vdQuote);

  TfrxValuedXMLReader = class(TfrxXMLReader)
  protected
    function IsLastSlash(const InSt: String): Boolean;
    function IsFirstSlash(const InSt: String): Boolean;
    function IsFirstNumberSign(const InSt: String): Boolean;
    procedure ReadValuedItem(out {$IFDEF Delphi12}NameS, ValueS{$ELSE}Name, Value{$ENDIF}, Text: String); virtual;
    procedure ProcessSecondLeftBrocket; virtual; abstract;
    function CreateItem: TfrxXMLItem; virtual;
    procedure AddChar(var st: AnsiString; var stPos: Integer; ch: AnsiChar);
  public
    function IsReadValuedXMLRootItem(Item: TfrxXMLItem): Boolean;
    function IsReadValuedXMLItem(Item: TfrxXMLItem): Boolean;
  end;

  TfrxValuedXMLDocument = class(TfrxXMLDocument)
  protected
    FValuedXMLStream: TStream;
    FValuedXMLStreamReader: TfrxValuedXMLReader;

    procedure ReadXMLHeader; virtual; abstract;
    function CreateReader: TfrxValuedXMLReader; virtual; abstract;
  public
    procedure InitValuedXMLFile(const FileName: String);
    procedure DoneValuedXMLFile;
    function IsReadItem(Item: TfrxXMLItem): Boolean;
  end;

  ESignException = class(Exception);

  TLogList = class(TStringList)
  public
    procedure Save(FileName: TFileName);
    procedure SaveAppend(FileName: TFileName);
  end;

function IncPointer(P: Pointer; Offset: Cardinal): Pointer;
function IntToAnsiStr(Value: Integer): AnsiString;
function ROL(const Value: LongWord; const Bits: Byte): LongWord;
function ROL16(const Value: Word; const Bits: Byte): Word;
function ROR16(const Value: Word; const Bits: Byte): Word;
function AnsiToHex(st: AnsiString; UpCase: Boolean = True): AnsiString;
function BufferToHex(Buffer: Pointer; Len: Cardinal; UpCase: Boolean = True): AnsiString;

function ByteSwap(Value: Cardinal): Cardinal; overload;
function ByteSwap(Value: Integer): Cardinal; overload;
function ByteSwap(Value: Int64): Int64; overload;

function AnsiStringOfChar(ACh: AnsiChar; Count: Integer): AnsiString;
function ToStringList(const S: string; const Delimiter: Char): TStrings;
function NormalizeScientificNotation(const S: string): string;

function Limit(const Value, MinValue, MaxValue: Single): Single; overload;
function Limit(const Value, MinValue, MaxValue: Double): Double; overload;
function Limit(const Value, MinValue, MaxValue: Integer): Integer; overload;

function ReadExtended8(Stream: TStream): Extended;
function ReadExtended10(Stream: TStream): Extended;
function ReadExtended16(Stream: TStream): Extended;

function ReadWideStringFromStream(Stream: TStream): WideString;
procedure WriteWideStringToStream(Stream: TStream; const ws: WideString);

implementation
end.

