
{******************************************}
{                                          }
{             FastReport VCL               }
{           Open Street Map File           }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxOSMFileFormat;

{$I frx.inc}

interface

uses
  frxXML, Classes, frxMapHelpers, frxAnaliticGeometry;

type
  TOSMNode = class(TTaggedElement)
  private
    FLatitude: Double;
    FLongitude: Double;
  protected
  public
    constructor Create(Lat, Lon: Double);

    property Latitude: Double read FLatitude;
    property Longitude: Double read FLongitude;
  end;

  TOSMWay = class(TTaggedElement)
  private
    function GetCount: integer;
    function GetShapeType: TShapeType;
    function GetNodes(Index: Integer): String;
  protected
    FNodeRefs: TStringList;
  public
    constructor Create;
    destructor Destroy; override;

    procedure AddNodeRef(const stNodeRef: String);

    property Count: integer read GetCount;
    property ShapeType: TShapeType read GetShapeType;
    property NodeRefs[Index: Integer]: String read GetNodes;
  end;

  TOSMFile = class
  private
    FXmin: Double;
    FYmin: Double;
    FXmax: Double;
    FYmax: Double;
    function GetCountOfNodes: Integer;
    function GetCountOfWays: Integer;
    function GetWays(Index: integer): TOSMWay;
    function GetNodes(Index: integer): TOSMNode;
    procedure LoadFrom(const FileName: string; Stream: TStream = nil);
  protected
    FOSMNodes: TStringList;
    FOSMWays: TStringList;
    FSumTags: TfrxSumStringList;
    FValidBounds: Boolean;
    procedure LoadBounds(XMLItem: TfrxXMLItem);
    procedure LoadNode(XMLItem: TfrxXMLItem);
    procedure LoadWay(XMLItem: TfrxXMLItem);
    procedure ParseItem(XMLItem: TfrxXMLItem);
    function ValidUTF8(st: String): String;
  public
    constructor Create(const FileName: string; Stream: TStream = nil); overload;
    destructor Destroy; override;
    function IsGetNodeAsPoint(const iWay, iNode: integer; out DP: TDoublePoint): boolean;
    function IsValidBounds: Boolean;

    property Xmin: Double read FXmin;
    property Ymin: Double read FYmin;
    property Xmax: Double read FXmax;
    property Ymax: Double read FYmax;

    property CountOfWays: Integer read GetCountOfWays;
    property Ways[Index: integer]: TOSMWay read GetWays;
    property CountOfNodes: Integer read GetCountOfNodes;
    property Nodes[Index: integer]: TOSMNode read GetNodes;

    property SumTags: TfrxSumStringList read FSumTags;
  end;

implementation
end.
