
{******************************************}
{                                          }
{             FastReport VCL               }
{             Poligon Template             }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxPolygonTemplate;

interface

{$I frx.inc}

uses
  SysUtils, Contnrs, Graphics,
  frxJSON, frxMapHelpers, frxAnaliticGeometry;

type
  TPointIndexByName = class
  private
    function GetIndexByName(Name: String): Integer;
    function GetNameBitmap(Index: Integer): TBitmap;
  protected
    FPointName: array of String;
    FNameBitmap: array of TBitmap;
  public
    constructor Create(JSON: TObject);
    destructor Destroy; override;
    function Count: Integer;

    property IndexByName[Name: String]: Integer read GetIndexByName;
    property NameBitmap[Index: Integer]: TBitmap read GetNameBitmap;
  end;

  TAbstractTemplateComponent = class
  protected
    FPointIndex: array of Integer;
    FPointIndexByName: TPointIndexByName; // Pointer to paren object
  public
    constructor Create(JSON: TfrxJSON; PointIndexByName: TPointIndexByName);
    procedure Draw(Canvas: TCanvas; Points: TPointArray); virtual; abstract;
  end;

  TTemplateComponentList = class(TObjectList)
  private
    function GetItems(Index: Integer): TAbstractTemplateComponent;
    procedure SetItems(Index: Integer; const Value: TAbstractTemplateComponent);
  public
    constructor Create;

    property Items[Index: Integer]: TAbstractTemplateComponent read GetItems write SetItems; default;
  end;

  TfrxPolygonTemplate = class
  private
    FName: String;
    FCount: Integer;
  protected
    FPointIndexByName: TPointIndexByName;
    FTemplateComponentList: TTemplateComponentList;
  public
    constructor Create(JSON: TfrxJSON);
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; Points: TPointArray; DrawNames: Boolean = False);
    procedure DrawWithNames(Canvas: TCanvas; Points: TPointArray);
    procedure LoadArray(JSON: TfrxJSON; nm: String);

    property Name: String read FName;
    property Count: Integer read FCount;
  end;

  TfrxPolygonTemplateList = class(TObjectList)
  private
    function GetTemplate(Index: Integer): TfrxPolygonTemplate;
    procedure SetTemplate(Index: Integer; const Value: TfrxPolygonTemplate);
    function GetItemsByName(Name: String): TfrxPolygonTemplate;
  protected
    FVersion: Integer;
  public
    constructor Create(FileName: TFileName);

    property Items[Index: Integer]: TfrxPolygonTemplate read GetTemplate write SetTemplate; default;
    property ItemsByName[Name: String]: TfrxPolygonTemplate read GetItemsByName;
  end;

var
  PolygonTemplateList: TfrxPolygonTemplateList;

implementation
end.
