
{******************************************}
{                                          }
{             FastReport VCL               }
{      Language resources management       }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxRes;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}Windows,{$ENDIF}
  SysUtils, Classes, Controls, Graphics, Forms, TypInfo, Buttons
  {$IFDEF FPC}
  , LResources, LCLProc, LCLType, LazHelper
  {$ELSE}
  , frxUnicodeUtils, ImgList
  {$ENDIF}
{$IFDEF Delphi6}
, Variants
{$ENDIF}
{$IFDEF FR_COM}
, ComObj
, FastReport_TLB
, DispatchablePersistent
{$ENDIF}
{$IFDEF Delphi10}
, WideStrings
{$ENDIF}
;

const
  frxDefToolImageWidth = 16;
  frxDefToolImageHeight = 16;

type
  TfrxResources = class(TObject)
  private
    FDisabledButtonImages: TImageList;
    FMainButtonImages: TImageList;
    FNames: TStringList;
    FObjectImages: TImageList;
    FPreviewButtonImages: TImageList;
    FValues: TWideStrings;// TStringList;
    FWizardImages: TImageList;
    FLanguages: TStringList;
    FImagesList: TStringList;
    FHelpFile: String;
    FCP: Cardinal;
    FImagesPPI: Integer;
    procedure BuildLanguagesList;
    function GetMainButtonImages: TImageList;
    function GetObjectImages: TImageList;
    function GetPreviewButtonImages: TImageList;
    function GetWizardImages: TImageList;
    procedure ResizeImages;
    procedure SetImagesPPI(Value: Integer);
    function GetImages(const aName: String; aPPIScale: Integer; aImageWidth: Integer = frxDefToolImageWidth; aImageHeight: Integer = frxDefToolImageHeight): TImageList;
    function GetDisabledButtonImages: TImageList;
    procedure AssignImages(Bitmap: TBitmap; dx, dy: Integer; ImgList1: TImageList; ImgList2: TImageList = nil);
  public
    constructor Create;
    destructor Destroy; override;
    function Get(const StrName: String): String;
    procedure Add(const Ref, Str: String);
    procedure AddW(const Ref: String; Str: WideString);
    procedure AddStrings(const Str: String);
    procedure AddXML(const Str: AnsiString);
    procedure Clear;
    procedure LoadFromFile(const FileName: String);
    procedure LoadFromStream(Stream: TStream);
    procedure SetButtonImages(Images: TBitmap; Clear: Boolean = False);
    procedure SetObjectImages(Images: TBitmap; Clear: Boolean = False);
    procedure SetPreviewButtonImages(Images: TBitmap; Clear: Boolean = False);
    procedure SetWizardImages(Images: TBitmap; Clear: Boolean = False);
    procedure SetGlyph(Button: TSpeedButton; Index: Integer);
    procedure UpdateFSResources;
    procedure Help(Sender: TObject); overload;
    property DisabledButtonImages: TImageList read GetDisabledButtonImages;
    property MainButtonImages: TImageList read GetMainButtonImages;
    property PreviewButtonImages: TImageList read GetPreviewButtonImages;
    property ObjectImages: TImageList read GetObjectImages;
    property WizardImages: TImageList read GetWizardImages;
    property Languages: TStringList read FLanguages;
    property HelpFile: String read FHelpFile write FHelpFile;
    property ImagesPPI: Integer read FImagesPPI write SetImagesPPI;
  end;

function frxResources: TfrxResources;
function frxGet(ID: Integer): String;


implementation
end.
