
{******************************************}
{                                          }
{             FastReport VCL               }
{               SVG Graphic                }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}
unit frxSVGGraphic;

interface

{$I frx.inc}

uses
  Windows, Classes, Graphics,
  frxSVGBase, frxSVGHelpers, frxHelpers;

type
  TfrxSVGGraphic = class(TGraphic)
  private
    FRootObj: TSVGRootObj;
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure ReadData(Stream: TStream); override;
    procedure WriteData(Stream: TStream); override;

    procedure Draw(ACanvas: TCanvas; const Rect: TRect); override;
    procedure DrawToSize(ACanvas: TCanvas; const Rect: TRect);

    function GetEmpty: Boolean; override;
    function GetWidth: Integer; override;
    function GetHeight: Integer; override;
    procedure SetHeight(Value: Integer); override;
    procedure SetWidth(Value: Integer); override;

    procedure AssignTo(Dest: TPersistent); override;

    procedure SetExternalParams(BoundsRect: TRect; ScaleX, ScaleY: Single; Centered: Boolean);
    procedure Clear;
    function IsAutoSize: Boolean;
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure LoadFromText(const Text: string);
    procedure LoadFromFile(const Filename: string); override;
    procedure LoadFromStream(Stream: TStream); override;
    procedure SaveToStream(Stream: TStream); override;

    procedure LoadFromClipboardFormat(AFormat: Word; AData: THandle;
      APalette: HPALETTE); override;
    procedure SaveToClipboardFormat(var AFormat: Word; var AData: THandle;
      var APalette: HPALETTE); override;
  end;

  TfrxSVGGraphicCache = class (TOwnObjList)
  protected
    function IsFindHash(MD5: AnsiString; out Index: Integer): Boolean;
  public
    function GraphicFromText(const Text: string): TGraphic;
    function GraphicFromFile(const Filename: string): TGraphic;
    function GraphicFromStream(Stream: TStream): TGraphic;
  end;

implementation
end.
