{******************************************}
{                                          }
{             FastReport VCL               }
{           Syntax memo control            }
{                                          }
{         Copyright (c) 1998-2021          }
{            by Fast Reports Inc.          }
{                                          }
{******************************************}

unit frxSynMemo;

interface

{$I frx.inc}

uses
  {$IFNDEF FPC}Windows, Messages, Imm,{$ENDIF}
  Types, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls, Forms,
  frxCtrls, fs_iparser, frxPopupForm, fs_xml, fs_iinterpreter, Menus, frxRes,
  frxHint
  {$IFDEF FPC}
  ,LCLType, LMessages, LazHelper, LCLIntf, LCLProc
  {$ENDIF};

const
  WM_FRX_SYNC_SCRIPT = WM_USER + 100;
  WM_FRX_UPDATE_CODE = WM_USER + 101;
  WM_FRX_FILL_CODE_COMPLETION = WM_USER + 102;

type
  TfrxCompletionList = class;
  TCharAttr = (caNo, caText, caComment, caKeyword, caString,
    caNumber);
  //TCharAttributes = set of TCharAttr;
  TfrxCharAttributes = record
    StyleIndex: Byte;
    IsSelBlock: Boolean;
  end;

  TfrxAttributeStyle = class(TCollectionItem)
  private
    FStyleID: Byte;
    FFontColor: TColor;
    FFontStyle: TFontStyles;
    FKeywords: TStrings;
    FAttrType: TCharAttr;
  protected
    procedure SetCollection(Value: TCollection); override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    property StyleID: Byte read FStyleID;
  published
    property AttrType: TCharAttr read FAttrType write FAttrType;
    property FontColor: TColor read FFontColor write FFontColor;
    property FontStyle: TFontStyles read FFontStyle write FFontStyle;
    property Keywords: TStrings read FKeywords;
  end;

  TfrxAttributeStyles = class(TCollection)
  private
    { null object }
    FDefaultAttribute: TfrxAttributeStyle;
    FIndexedList: TStringList;
    function GetItem(Index: Integer): TfrxAttributeStyle;
  public
    function GetUniqueID: Byte;
    constructor Create;
    destructor Destroy; override;
    function Add: TfrxAttributeStyle;
    function GetStyleByID(ID: Byte): TfrxAttributeStyle;
    procedure AssignStyleByID(ID: Byte; aFont: TFont);
    procedure AssignStyle(aStyle: TfrxAttributeStyle; aFont: TFont);
    function FindStyleIDByKeyword(const Name: String; AttrType: TCharAttr): Byte;
    property Items[Index: Integer]: TfrxAttributeStyle read GetItem; default;
  end;

  TfrxSynDialectStyle = class(TPersistent)
  private
    FAttributeStyles: TfrxAttributeStyles;
    FKeywords: String;
    FCommentLine1: String;
    FCommentLine2: String;
    FCommentBlock1: String;
    FCommentBlock2: String;
    FStringQuotes: String;
    FHexSequence: String;
    FName: String;
    FOwner: TObject;
    procedure SetName(const Value: String);
  public
    constructor Create(AOwner: TObject);
    destructor Destroy; override;
    { TODO: we need one contract for all setings. I.e. need interface for this }
    procedure SaveTo(const Section: String; Ini: TObject);
    procedure LoadFrom(const Section: String; Ini: TObject);
    property AttributeStyles: TfrxAttributeStyles read FAttributeStyles;
    property Name: String read FName write SetName;
  published
    property Keywords: String read FKeywords write FKeywords;
    property CommentLine1: String read FCommentLine1 write FCommentLine1;
    property CommentLine2: String read FCommentLine2 write FCommentLine2;
    property CommentBlock1: String read FCommentBlock1 write FCommentBlock1;
    property CommentBlock2: String read FCommentBlock2 write FCommentBlock2;
    property StringQuotes: String read FStringQuotes write FStringQuotes;
    property HexSequence: String read FHexSequence write FHexSequence;
  end;

  TfrxSynDialectStyles = class(TObject)
  private
    FLIst: TList;
    FDefSynDialectStyle: TfrxSynDialectStyle;
    FActiveIndex: Integer;
    FActiveChanged: TNotifyEvent;
    function GetItem(Index: Integer): TfrxSynDialectStyle;
    function GetActiveDialicet: TfrxSynDialectStyle;
    function GetActiveStyles: TfrxAttributeStyles;
    procedure SetActiveIndex(const Value: Integer);
   public
    constructor Create;
    destructor Destroy; override;
    function Add: TfrxSynDialectStyle;
    procedure Clear;
    function Count: Integer;
    { TODO: we need one contract for all setings. I.e. need interface for this }
    procedure SaveTo(const Section: String; Ini: TObject);
    procedure LoadFrom(const Section: String; Ini: TObject);
    property Items[Index: Integer]: TfrxSynDialectStyle read GetItem; default;
    property ActiveDialicet: TfrxSynDialectStyle read GetActiveDialicet;
    property ActiveStyles: TfrxAttributeStyles read GetActiveStyles;
    property ActiveIndex: Integer read FActiveIndex write SetActiveIndex;
    property OnActiveChanged: TNotifyEvent read FActiveChanged write FActiveChanged;
  end;

  TfrxCodeCompletionEvent = procedure(const Name: String; List: TfrxCompletionList) of object;

  TfrxByteArr = array of Byte;

  TfrxSynAttributes = class
  private
    FArray: array of TfrxByteArr;
    FCapacity: Integer;
    FCount: Integer;
    FUpdating: Boolean;
    FDialectStyles: TfrxSynDialectStyles;
    FParser: TfsParser;
    procedure SetCapacity(NewCapacity: Integer);
    procedure SetCount(NewCount: Integer);
    function GetLine(Index: Integer): TfrxByteArr;
    procedure PutLine(Index: Integer; const Value: TfrxByteArr);
    procedure ActiveChanged(Sender: TObject);
  public
    constructor Create(Parser: TfsParser; DialectStyles: TfrxSynDialectStyles);
    procedure SetLineLen(Index: Integer; NewLen: Integer);
    procedure Delete(Index: Integer);
    function GetAllAttributes: TfrxByteArr;
    procedure SetAllAttributes(Attr: TfrxByteArr);
    procedure UpdateSyntax(EndLine: Integer; Text: TStringList);
    procedure UpdateSyntaxDialect;
    property Line[Index: Integer]: TfrxByteArr read GetLine write PutLine; default;
    property Count: Integer read FCount write SetCount;
    property Updating: Boolean read FUpdating;
  end;

  TfrxBreakPoint = class
  private
    FCondition: String;
    FSpecialCondition: String;
    FEnabled: Boolean;
    FLine: Integer;
  public
    property Condition: String read FCondition write FCondition;
    property Enabled: Boolean read FEnabled write FEnabled;
    property Line: Integer read FLine write FLine;
    property SpecialCondition: String read FSpecialCondition write FSpecialCondition;
  end;

  TfrxItemType = (itVar, itProcedure, itFunction, itProperty, itIndex, itConstant, itConstructor, itType, itEvent);
  TfrxItemTypes = set of TfrxItemType;

  TfrxCompletionListType = (cltRtti, cltScript, cltAddon);
  TfrxCompletionListTypes = set of TfrxCompletionListType;

  TfrxCompletionItem = class
  private
    FParent: TfrxCompletionItem;
    FName: String;
    FType: String;
    FParams: String;
    FItemType: TfrxItemType;
    FStartVisible: Integer;
    FEndVisible: Integer;
  public
    property Name: String read FName;
    property Typ: String read FType;
    property Params: String read FParams;
    property ItemType: TfrxItemType read FItemType;
  end;

  TfrxCompletionList = class
  private
    FConstants: TStringList;
    FVariables: TStringList;
    FFunctions: TStringList;
    FClasses: TStringList;
    FLocked: Boolean;
    function AddBaseVar(varList: TStrings; const Name, sType: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function GetItem(Index: Integer): TfrxCompletionItem;
  public
    constructor Create;
    destructor Destroy; override;
    procedure DestroyItems;
    function Count: Integer;
    function AddConstant(const Name, sType: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function AddVariable(const Name, sType: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function AddClass(const Name, sType: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function AddFunction(const Name, sType, Params: String; VisibleStart: Integer = 0; VisibleEnd: Integer = -1; const ParentFunc: String = ''): TfrxCompletionItem;
    function Find(const Name: String): TfrxCompletionItem;
    property Items[Index: Integer]: TfrxCompletionItem read GetItem; default;
    property Locked: Boolean read FLocked write FLocked;
  end;

  { TfrxCodeCompletionThread }

  TfrxCodeCompletionThread = class(TThread)
  private
    FText: TStringList;
    FScript: TfsScript;
    FOriginalScript: TfsScript;
    FILCode: TStream;
    FXML: TfsXMLDocument;
    FCompletionList: TfrxCompletionList;
    FSyntaxType: String;
    FMemoHandle: HWND;
    procedure SyncScript;
    procedure UpdateCode;
    procedure FreeScript;
    procedure FillCodeCompletion;
  public
    destructor Destroy; override;
    procedure Execute; override;
    property Script: TfsScript read FOriginalScript write FOriginalScript;
  end;

  {$IFDEF FPC}
  { TBrkStringList }

  TBrkStringList = class(TStringList)
    function AddObject(const S: string; AObject: TObject): Integer; override; overload;
  end;
  {$ENDIF}

  TfrxSyntaxMemo = class(TfrxScrollWin)
  private
    {$IFDEF NONWINFPC}
    FCaretCreated: Boolean;
    {$ENDIF}
    FActiveLine: Integer;
    FAllowLinesChange: Boolean;
    FBlockColor: TColor;
    FBlockFontColor: TColor;
    FBookmarks: array[0..9] of Integer;
    FCharHeight: Integer;
    FCharWidth: Integer;
    FCommentAttr: TFont;
    FCompletionForm: TfrxPopupForm;
    FCompletionLB: TListBox;
    FDoubleClicked: Boolean;
    FDown: Boolean;
    FToggleBreakPointDown: Boolean;
    FGutterWidth: Integer;
    FIsMonoType: Boolean;
    FKeywordAttr: TFont;
    FMaxLength: Integer;
    FMessage: String;
    FModified: Boolean;
    FMoved: Boolean;
    FNumberAttr: TFont;
    FOffset: TPoint;
    FOnChangePos: TNotifyEvent;
    FOnChangeText: TNotifyEvent;
    FOnCodeCompletion: TfrxCodeCompletionEvent;
    FParser: TfsParser;
    FPos: TPoint;
    FStringAttr: TFont;
    FSelEnd: TPoint;
    FSelStart: TPoint;
    FTabStops: Integer;
{$IFNDEF FPC}
    FCompSelStart: TPoint;
    FCompSelEnd: TPoint;
{$ENDIF}
    FShowGutter: boolean;
    //FSynStrings: TStrings;
    FSynAttributes: TfrxSynAttributes;
    FSynDialectStyles: TfrxSynDialectStyles;
    FSyntax: String;
    FTempPos, FLastHintPos: TPoint;
    FText: TStringList;
    FTextAttr: TFont;
    FUndo: TStringList;
    FWindowSize: TPoint;
    FBreakPoints: {$IFDEF FPC}TBrkStringList{$ELSE}TStringList{$ENDIF};
    FCodeCompList: TStringList;
    FStartCodeCompPos: Integer;
    FCompleationFilter: String;
    FScriptRTTIXML: TfsXMLDocument;
    FRttiCompletionList: TfrxCompletionList;
    FScriptCompletionList: TfrxCompletionList;
    FAddonCompletionList: TfrxCompletionList;
    FClassCompletionList: TfrxCompletionList;
    FCodeCompletionThread: TfrxCodeCompletionThread;
    FScript: TfsScript;
    FTimer: TTimer;
    FShowLineNumber: Boolean;
    FCodeComplitionFilter: TfrxItemTypes;
    FShowInCodeComplition: TfrxCompletionListTypes;
    FCodeCompletionWidth: Integer;
    FCodeCompletionHeight: Integer;
    FCodeCompletionMinHeight: Integer;
    FCodeCompletionMinWidth: Integer;
    FMultiByteLang: Boolean;
    FfrxHintShowEvent: TfrxHintShowEvent;
    FEnableHint: Boolean;
{$IFNDEF FPC}
    FTmpCanvas: TBitmap;
    {  need for east languages carret correction  }
    function GetCharWidth(const Str: String): Integer;
    function GetCharXPos(X: Integer): Integer;
{$ENDIF}
    function GetCharAttr(Pos: TPoint): TfrxCharAttributes;
    function GetLineBegin(Index: Integer): Integer;
    function GetPlainTextPos(Pos: TPoint): Integer;
    function GetPosPlainText(Pos: Integer): TPoint;
    function GetRunLine(Index: Integer): Boolean;
    function GetSelText: String;
    function GetText: TStrings;
    function LineAt(Index: Integer; UseTrim: Boolean = True): String;
    function GetIdentEnd(aPos: Integer): Integer;
    function IsCursorInStringBlock: Boolean;
    function LineLength(Index: Integer): Integer;
    function Pad(n: Integer): String;
    procedure AddSel;
    procedure AddUndo;
    procedure ClearSel;
    procedure ClearSyntax(ClearFrom: Integer);
    procedure CompletionFormClose(Sender: TObject; var Action: TCloseAction);
    procedure CompletionLBDblClick(Sender: TObject);
    procedure CompletionLBDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState);
    function ItemToPrefix(Item: TfrxCompletionItem; var pref: String; c: TColor = clNone): TColor;
    procedure CompletionLBKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure CompletionLBKeyPress(Sender: TObject; var Key: Char);
    procedure CorrectBookmark(Line, Delta: Integer);
    procedure CorrectBreakPoints(Line, Delta: Integer);
    procedure CreateSynArray(EndLine: Integer);
    procedure DoBackspace;
    procedure DoChange;
    procedure DoChar(Ch: Char);
    procedure DoCodeCompletion;
    procedure BuildCClist(const sName: String; aCodeCompList: TStringList; AllFunc: Boolean = False);
    procedure DoCtrlI;
    procedure DoCtrlU;
    procedure DoCtrlR;
    procedure DoCtrlL;
    procedure DoDel;
    procedure DoDown;
    procedure DoEnd(Ctrl: Boolean);
    procedure DoHome(Ctrl: Boolean);
    procedure DoLeft;
    procedure DoPgUp;
    procedure DoPgDn;
    procedure DoReturn;
    procedure DoRight;
    procedure DoUp;
    procedure EnterIndent;
    procedure LinesChange(Sender: TObject);
    procedure SetActiveLine(Line: Integer);
    procedure SetCommentAttr(Value: TFont);
    procedure SetKeywordAttr(Value: TFont);
    procedure SetNumberAttr(const Value: TFont);
    procedure SetRunLine(Index: Integer; const Value: Boolean);
    procedure SetSelText(const Value: String);
    procedure SetShowGutter(Value: Boolean);
    procedure SetStringAttr(Value: TFont);
    procedure SetSyntax(const Value: String);
    procedure SetText(Value: TStrings);
    procedure SetTextAttr(Value: TFont);
    procedure ShiftSelected(ShiftRight: Boolean);
    procedure ShowCaretPos;
    procedure TabIndent;
    procedure UnIndent;
    procedure UpdateScrollBar;
    procedure WMKillFocus(var Msg: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMSetFocus(var Msg: TWMSetFocus); message WM_SETFOCUS;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    { Inline IME realisation }
	{$IFNDEF FPC}
    procedure WMIMEStartComp(var Message: TMessage); message WM_IME_STARTCOMPOSITION;
    procedure WMIMEEndComp(var Message: TMessage); message WM_IME_ENDCOMPOSITION;
    procedure WMIMECOMPOSITION (var Message: TMessage); message WM_IME_COMPOSITION ;
	{$ENDIF}
    procedure WMFRXSyncScript(var Message: TMessage); message WM_FRX_SYNC_SCRIPT;
    procedure WMFRXUpdateCode(var Message: TMessage); message WM_FRX_UPDATE_CODE;
    procedure WMFRXFillCodeCompletion(var Message: TMessage); message WM_FRX_FILL_CODE_COMPLETION;
    procedure SetEnableHint(vEnableHint: boolean);
    procedure FakeHintShow(var Msg: TCMHintShow);
    procedure RealHintShow(var Msg: TCMHintShow);
    procedure WMFRXHintShow(var Msg: TCMHintShow); message CM_HINTSHOW;
    function OffsPoint(Point: TPoint): TPoint;
    procedure GetCurWord(Point: TPoint; var Prefix: String; var Header: String; var TextHint: String);
    function CurPosToSynPos(X, Y: Integer): TPoint;
    function GetTextSelected: Boolean;
    procedure SetGutterWidth(const Value: Integer);
    procedure SetShowInCodeComplition(const Value: TfrxCompletionListTypes);
    procedure SetCodeCompletionWidth(const Value: Integer);
    procedure SetCodeCompletionHeight(const Value: Integer);
    procedure SetCodeCompletionMinHeight(const Value: Integer);
    procedure SetCodeCompletionMinWidth(const Value: Integer);
    procedure PopCopy(Sender: TObject);
    procedure PopCut(Sender: TObject);
    procedure PopPaste(Sender: TObject);
    procedure PopDelete(Sender: TObject);
    procedure PopSelectAll(Sender: TObject);
    function GetAttributeStyles: TfrxAttributeStyles;
    procedure QuickComment;
  protected
    procedure DblClick; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    {$IFDEF FPC}
    procedure UTF8KeyPress(var UTF8Key: TUTF8Char); override;
    {$ENDIF}
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure MouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure OnHScrollChange(Sender: TObject); override;
    procedure OnVScrollChange(Sender: TObject); override;
    procedure Resize; override;
    function GetCompletionString(Pos: TPoint): String;
    function GetFilter(aStr: String): String;
    procedure DoTimer(Sender: TObject);
    procedure DoPPIChanged(aNewPPI: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure CopyToClipboard;
    procedure CutToClipboard;
    procedure CompletionClose;
    procedure PasteFromClipboard;
    procedure SelectAll;
    procedure SetPos(x, y: Integer);
    procedure SetPosByCoords(X, Y: Integer);
    procedure ShowMessage(const s: String);
    procedure Undo;
    procedure UpdateView;
    procedure ClearBreakPoints;
    function Find(const SearchText: String; CaseSensitive: Boolean;
      var SearchFrom: Integer): Boolean;
    function GetPlainPos: Integer;
    function GetPos: TPoint;
    function IsBookmark(Line: Integer): Integer;
    procedure AddBookmark(Line, Number: Integer);
    procedure DeleteBookmark(Number: Integer);
    procedure GotoBookmark(Number: Integer);
    procedure AddNewBreakPoint;
    procedure AddBreakPoint(Number: Integer; const Condition: String; const Special: String);
    procedure ToggleBreakPoint(Number: Integer; const Condition: String);
    procedure DeleteBreakPoint(Number: Integer);
    procedure DeleteF4BreakPoints;
    function IsBreakPoint(Number: Integer): Boolean;
    function IsActiveBreakPoint(Number: Integer): Boolean;
    function GetBreakPointCondition(Number: Integer): String;
    function GetBreakPointSpecialCondition(Number: Integer): String;
    procedure FillRtti;
    procedure SaveToIni(const IniPath: String; const Section: String; const FileName: String);
    procedure LoadFromIni(const IniPath: String; const Section: String; const FileName: String);

//    property DefaultAttributeStyles: TfrxAttributeStyles read FAttributeStyles;
    property EnableHint: Boolean read FEnableHint write SetEnableHint default True;
    property AttributeStyles: TfrxAttributeStyles read GetAttributeStyles;
    property CodeCompletionThread: TfrxCodeCompletionThread read FCodeCompletionThread;
    property CodeCompletionMinWidth: Integer read FCodeCompletionMinWidth write SetCodeCompletionMinWidth default 300;
    property CodeCompletionMinHeight: Integer read FCodeCompletionMinHeight write SetCodeCompletionMinHeight default 100;
    property CodeCompletionWidth: Integer read FCodeCompletionWidth write SetCodeCompletionWidth default 300;
    property CodeCompletionHeight: Integer read FCodeCompletionHeight write SetCodeCompletionHeight default 100;
    property ActiveLine: Integer read FActiveLine write SetActiveLine;
    property BlockColor: TColor read FBlockColor write FBlockColor;
    property BlockFontColor: TColor read FBlockFontColor write FBlockFontColor;
    property BreakPoints:{$IFDEF FPC}TBrkStringList{$ELSE}TStringList{$ENDIF} read FBreakPoints;
    property Color;
    property CommentAttr: TFont read FCommentAttr write SetCommentAttr;
    property CodeComplitionFilter: TfrxItemTypes read FCodeComplitionFilter write FCodeComplitionFilter;
    property ShowInCodeComplition: TfrxCompletionListTypes read FShowInCodeComplition write SetShowInCodeComplition;
    property SynDialectStyles: TfrxSynDialectStyles read FSynDialectStyles;
    property Font;
	{$IFNDEF FPC}
    property ImeMode;
    property ImeName;
  	{$ENDIF}
    property SelStart: TPoint read FSelStart write FSelStart;
    property SelEnd: TPoint read FSelEnd write FSelEnd;
    property TabStops: Integer read FTabStops write FTabStops;
    property GutterWidth: Integer read FGutterWidth write SetGutterWidth;
    property KeywordAttr: TFont read FKeywordAttr write SetKeywordAttr;
    property Modified: Boolean read FModified write FModified;
    property MultiByteLang: Boolean read FMultiByteLang write FMultiByteLang;
    property NumberAttr: TFont read FNumberAttr write SetNumberAttr;
    property RunLine[Index: Integer]: Boolean read GetRunLine write SetRunLine;
    property SelText: String read GetSelText write SetSelText;
    property StringAttr: TFont read FStringAttr write SetStringAttr;
    property TextAttr: TFont read FTextAttr write SetTextAttr;
    property Lines: TStrings read GetText write SetText;
    property Syntax: String read FSyntax write SetSyntax;
    property Script: TfsScript read FScript write FScript;
    property ShowGutter: Boolean read FShowGutter write SetShowGutter;
    property ShowLineNumber: Boolean read FShowLineNumber write FShowLineNumber;
    property TextSelected: Boolean read GetTextSelected;
    property ScriptRTTIXML: TfsXMLDocument read FScriptRTTIXML;
    property OnChangePos: TNotifyEvent read FOnChangePos write FOnChangePos;
    property OnChangeText: TNotifyEvent read FOnChangeText write FOnChangeText;
    property OnCodeCompletion: TfrxCodeCompletionEvent read FOnCodeCompletion
      write FOnCodeCompletion;
    property OnDragDrop;
    property OnDragOver;
    property OnKeyDown;
    property PopupMenu;
  end;

  TDiffFunc = function(s1, s2: String): Integer;

implementation
end.
